import org.graphstream.algorithm.Dijkstra;
import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.implementations.SingleGraph;

import java.util.*;

import static org.graphstream.algorithm.Dijkstra.Element.EDGE;
import static org.graphstream.algorithm.Dijkstra.Element.NODE;

public class Djikstra{

    HashMap<Node, Double> queue = new HashMap<Node, Double>();

    public Graph generer_Aleatoire(int taille) {

        Graph graph = new SingleGraph("Graphe aléatoire");
        System.setProperty("org.graphstream.ui.renderer", "org.graphstream.ui.j2dviewer.J2DGraphRenderer");
        graph.addAttribute("ui.quality");
        graph.addAttribute("ui.antialias");
        Generator g = new RandomGenerator(10,false, false,"","cap");
        //g.setEdgeAttributesRange(10.0, 50.0);
        //g.addEdgeAttribute("cap");
        g.addSink(graph);
        g.begin();

        for (int i = 0; i < taille; i++)
            g.nextEvents();
        g.end();
        return graph;
    }
    private void init(Graph graph, Node source) {
        for (Node n : graph.getEachNode()) {
            n.addAttribute("dist", n.getId().equals(source.getId()) ? 0 : Double.POSITIVE_INFINITY);
            n.addAttribute("parent", new Stack<Node>());
        }
    }


    //METHODE DJIKSTRA DU COURS
    public void djikstraNaive(Graph graph, Node source){
        long debut, fin;
        debut = System.currentTimeMillis();
        init(graph, source);
        queue.put(source, source.getAttribute("dist"));
        while (queue.size() != 0) {
            Node courant = prioriteMin(queue.keySet());
            Iterator<Edge> edgeIterator = courant.getLeavingEdgeIterator();
            queue.remove(courant);
            //System.out.println("" + currNode.getId() + " " + currNode.getAttribute("dist"));
            // parcours des voisins
            while (edgeIterator.hasNext()) {
                Edge currEdge = edgeIterator.next();
                Node opNode = currEdge.getTargetNode();
                if(! queue.containsKey(opNode)) {
                    Double currNodeDist = courant.getAttribute("dist");
                    Double edgeWeight = currEdge.getAttribute("cap");
                    Double sum = currNodeDist + edgeWeight;
                    Double opNodeDistance = opNode.getAttribute("dist");
                    if (opNodeDistance > sum) {
                        opNode.setAttribute("dist", sum);
                        // reste plus
                        opNode.setAttribute("parent", courant);
                        queue.put(opNode, opNode.getAttribute("dist"));
                    }
                }
                System.out.println("Noeud " + graph.getNode(0) + "->" + opNode.getId() + " " + opNode.getAttribute("dist"));
            }
        }
        fin = System.currentTimeMillis();
        long temps = fin - debut;
        System.out.println("Le temps d'éxécution de la méthode naÏve : " +  temps + "ms");


    }

    private Node prioriteMin(Set<Node> nodes) {
        Node plusPetite = null;
        Double plusPetiteDist = Double.MAX_VALUE;
        for (Node node : nodes) {
            Double distance = node.getAttribute("dist");
            if (distance < plusPetiteDist) {
                plusPetiteDist = distance;
                plusPetite = node;
            }
        }
        return plusPetite;
    }


    public void djikstraPCC(Graph graph) {
       long debut = System.currentTimeMillis();

        Dijkstra dijkstra = new Dijkstra(EDGE, null, null);

        dijkstra.init(graph);
        dijkstra.setSource(graph.getNode("0"));
        dijkstra.compute();
        //System.out.println(dijkstra.getPath(graph.getNode(String.valueOf(sommet2))));

        /*System.out.printf("%s->%s:%6.2f%n", dijkstra.getSource(), sommet2, dijkstra.getPathLength(sommet2));
        for (Edge edge : dijkstra.getPathEdges(sommet2)) edge.addAttribute("ui.style", "fill-color: red;");*/

        for (Node node : graph) {
             System.out.printf("%s->%s:%6.2f%n", dijkstra.getSource(), node, dijkstra.getPathLength(node));
        }
        long fin = System.currentTimeMillis();
        long temps = fin - debut;
        System.out.println("Le temps d'éxécution de la méthode djikstra GraphStream : " +  temps + "ms");

    }

}
